# -*- coding: utf-8 -*-
from django import template

register = template.Library()

@register.simple_tag(takes_context=True)
def subscribe_form(context):
    "Get an empty subscription form. Added form context variable is only valid within the block scope."
    from ..forms import SubscribeForm
    context['form'] = SubscribeForm(request=context['request'])
    context['action'] = 'DO_NOTHING'
    return ''

@register.simple_tag(takes_context=True)
def confirm_form(context):
    "Get an empty confirmation form. Added form context variable is only valid within the block scope."
    from ..forms import ConfirmForm_factory
    context['form'] = ConfirmForm_factory()(request=context['request'])
    context['action'] = 'DO_NOTHING'
    return ''



# -*- coding: utf-8 -*-
from rest_framework.renderers import TemplateHTMLRenderer
from rest_framework import generics, status
from rest_framework.response import Response
from .forms import SubscribeForm, ConfirmForm_factory
from .serializers import SubscribeSerializer, ConfirmSerializer_factory


class SubscribeView(generics.CreateAPIView):
    """Only allows posts to create new customers or add the email address"""
    # for debugging via the DRF browsable api
    serializer_class = SubscribeSerializer

    def create(self, request):
        customer_form = SubscribeForm(data=request.data, request=request)

        if customer_form.is_valid():
            # if valid, customer_form will assign the email address to the customer
            # and an email will be sent
            customer_form.save()
            return Response(request.data, status=status.HTTP_201_CREATED) 
        else:
            return Response({'errors': customer_form.errors}, status=status.HTTP_400_BAD_REQUEST)


class ConfirmView(generics.UpdateAPIView):
    """
    PUT and PATCH methods for submitting new subscriptions
    Retrieve API not necessary as form will be rendered with params
    from the url
    Also supports GET to render a default form.
    """
    # for debugging via the DRF browsable api
    serializer_class = ConfirmSerializer_factory()
    renderer_classes = [TemplateHTMLRenderer] + generics.UpdateAPIView.renderer_classes
    template_name = "shop_subscribe/default-confirm-form.html"
    form_class = ConfirmForm_factory()

    def get(self, request):
        """For the default HTML confirm form template"""
        form = self.form_class(request=request)
        # fix to allow browsable api and template renderer
        if request.query_params.get('format', None) in ['json', 'api']:
            # form.initial contains the model data overridden by any initial data passed in
            context = form.initial
        else:
            # we assume 'html'
            context = {
                'form': form,
                'action': 'DO_NOTHING'
            }
        return Response(context)

    def update(self, request, *args, **kwargs):
        """PUT and PATCH go here"""
        customer_form = self.form_class(data=request.data, request=request)

        if customer_form.is_valid():
            customer_form.save()
            return Response(request.data) 
        else:
            return Response({'errors': customer_form.errors}, status=status.HTTP_400_BAD_REQUEST)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from setuptools import setup, find_packages
import shop_subscribe

with open('README.rst') as fd:
    README = fd.read()

CLASSIFIERS = [
    'Environment :: Web Environment',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3.5',
    'Topic :: Software Development :: Libraries :: Application Frameworks',
    'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
]

setup(
    author="Richard Case",
    author_email="rich@racitup.com",
    name="djangoshop-subscribe",
    packages=find_packages(exclude=['doc']),
    version=shop_subscribe.__version__,
    description="An email subscription plugin for Django-SHOP",
    long_description=README,
    url='https://github.com/racitup/djangoshop-subscribe',
    license='BSD License',
    platforms=['OS Independent'],
    classifiers=CLASSIFIERS,
    keywords= ['Django', 'Django-SHOP'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'Django>=1.10.0,<1.11',
        'django-shop>=0.11',
        'django-post-office>3.0.0',
    ],
)



# -*- coding: utf-8 -*-
from collections import OrderedDict
from datetime import datetime
import re, logging
from django.core.signing import Signer, BadSignature
from django.template.loader import select_template
from django.contrib.sites.shortcuts import get_current_site
from django.utils.translation import get_language_from_request
from django.utils import timezone
from django.core.urlresolvers import reverse
from django.core.exceptions import ObjectDoesNotExist
from django.urls import NoReverseMatch
from cms.cache.page import get_page_url_cache, set_page_url_cache
from cms.templatetags.cms_tags import _get_page_by_untyped_arg
#from cms.utils import get_language_from_request
from rest_framework.request import Request as DRFRequest
from ipware.ip import get_real_ip, get_ip
from shop.conf import app_settings
from shop.models.customer import CustomerModel
from post_office import mail
from post_office.models import EmailTemplate


# Get an instance of a logger
logger = logging.getLogger('shop_subscribe')

sep = ':'
signer = Signer(sep=sep, salt='subscribe')
def sign(email):
    "Returns [email, signature]"
    return signer.sign(email).split(sep, 1)

def unsign(context):
    "Returns an ordereddict of email & signature, {} otherwise"
    sigcontext = OrderedDict([
        ('email', context.get('email', '')),
        ('sig', context.get('sig', ''))
    ])
    signer.unsign(sep.join( sigcontext.values() ))
    return sigcontext


def get_subscription_fields():
    """Returns the list of customer subscription fields"""
    return [item for item in dir(CustomerModel) if item.startswith('subscription_')]


def get_customer_from_emailsignature(request):
    """
    DRF or WSGI requests
    Validate the email signature in either the GET url for initial email link or POST hidden data for form submissions.
    If the signature is valid return a 'recognized' customer object if not already.
    """
    if isinstance(request, DRFRequest):
        try:
            # e.g. GET URLs
            context = unsign(request.query_params)
        except BadSignature:
            # e.g. POST data
            context = unsign(request.data)
    else:
        try:
            context = unsign(request.GET)
        except BadSignature:
            context = unsign(request.POST)

    customer, created = CustomerModel.objects.get_or_create_from_email(request, context['email'])
    # make the customer guest if not already
    if not customer.is_recognized():
        customer.recognize_as_guest()
    # remove sensitive data since the customer has confirmed
    customer.extra.pop('subscription_IP', None)
    customer.extra.pop('subscription_date', None)
    customer.last_access = timezone.now()
    customer.save()

    if created:
        logger.warning('Customer recreated from confirmation: %s' % context['email'])

    return customer, context


def reverse_cms_url(request, page_lookup):
    """Equivalent of CMS 'page_url' templatetag"""
    site_id = get_current_site(request).id
    lang = get_language_from_request(request)

    url = get_page_url_cache(page_lookup, lang, site_id)
    if url is None:
        try:
            page = _get_page_by_untyped_arg(page_lookup, request, site_id)
        except ObjectDoesNotExist:
            page = None
        if page:
            url = page.get_absolute_url(language=lang)
            set_page_url_cache(page_lookup, lang, site_id, url)
    if url:
        return url
    raise NoReverseMatch("CMS page not found")

def build_confirm_url(request, email='', sig=''):
    """
    Build the confirm url from supplied parameters
    """
    try:
        url = reverse_cms_url(request, 'shop-subscribe-confirm')
    except NoReverseMatch:
        try:
            url = reverse('shop-subscribe-confirm')
        except NoReverseMatch:
            url = reverse('shop_subscribe:confirm')
    url = url + '?email=' + '&sig='.join( (email, sig) )
    return request.build_absolute_uri(url)

def unisoformat(datestring):
    "Bit of a hacky way to get the date back from python's isoformat string"
    return datetime(*map(int, re.findall('\d+', datestring)))


_et_name = 'Subscription confirmation - customer'
def get_emailtemplate():
    """
    Gets or creates a post_office email template
    Returns the emailtemplate instance
    """
    try:
        et = EmailTemplate.objects.get(name=_et_name, language='', default_template=None)
    except EmailTemplate.DoesNotExist:
        subject = select_template([
            '{}/shop_subscribe/email/subscription-confirm-subject.txt'.format(app_settings.APP_LABEL),
            'shop_subscribe/email/subscription-confirm-subject.txt',
        ])
        html = select_template([
            '{}/shop_subscribe/email/subscription-confirm-body.html'.format(app_settings.APP_LABEL),
            'shop_subscribe/email/subscription-confirm-body.html',
        ])
        et = EmailTemplate(
            name=_et_name,
            description='Send the customer an email with a link to confirm their subscription',
            subject=''.join(subject.origin.loader.get_contents(subject.origin).splitlines()).strip(),
            html_content=html.origin.loader.get_contents(html.origin),
        )
        et.save()
    return et

def send_confirmation_email(request, customer):
    """
    Sends direct to post_office
    Assumes customer will be saved afterward externally
    """
    # check that same IP is not making lots of subscriptions and store IP
    ip = get_ip(request)
    now = datetime.now()
    for c in CustomerModel.objects.exclude(extra__exact='{}').all():
        try:
            if (c.extra['subscription_IP'] == ip and
                (now - unisoformat(c.extra['subscription_date'])).days < 1):
                logger.warning('Subscription from {} dropped. Same IP ({}) subscribed recently.'.format(customer.email, ip))
                return False
        except KeyError:
            continue

    context = {
        'site_name': get_current_site(request).name,
        'confirm_url': build_confirm_url(request, *sign(customer.email)),
        'email': customer.email,
        # requires django-ipware; only returns public IPs
        'ip': get_real_ip(request),
        'user_agent': request.META['HTTP_USER_AGENT'],
        'language': get_language_from_request(request, check_path=True)
    }

    mail.send(
        customer.email,
        template=get_emailtemplate(),
        context=context,
        render_on_delivery=True,
    )

    customer.extra.update({
        'subscription_IP': ip,
        'subscription_date': now.isoformat(),
    })
    return True



# -*- coding: utf-8 -*-
from django.contrib.auth import get_user_model
from django.core.exceptions import MultipleObjectsReturned
from shop.models.customer import CustomerModel


class SubscriptionCustomerManagerMixin(object):
    """
    Add method for subscriptions to lookup the customer by email address
    """
    def get_or_create_from_email(self, request, email):
        customers = self.filter(user__email=email)
        created = False
        try:
            customer = customers.get()
        except MultipleObjectsReturned:
            # find the best option and delete the others
            customers = customers.order_by('-recognized', '-last_access', '-pk')
            customer = customers[:1].get()
            for c in customers[1:]:
                c.delete()
        except CustomerModel.DoesNotExist:
            # make sure the user does not exist
            users = get_user_model().objects.filter(email=email)
            try:
                user = users.get()
            except MultipleObjectsReturned:
                # find the best option and delete the others
                users = users.order_by('-is_superuser', '-is_staff', '-is_active', '-last_login', '-date_joined', '-pk')
                user = users[:1].get()
                for u in users[1:]:
                    u.delete()
            except get_user_model().DoesNotExist:
                # our db has somehow lost the customer and their cart
                # or this is a new user with a cart
                if (not request.customer.is_visitor() and
                    request.customer.email in (email, '', None)):
                    # customer and user is in the db
                    request.customer.email = email
                    request.customer.save()
                    return request.customer, created
                else:
                    # create a fresh user like get_or_create_from_request()
                    request.session.cycle_key()
                    assert request.session.session_key
                    username = self.encode_session_key(request.session.session_key)
                    user = get_user_model().objects.create_user(username)
                    user.is_active = False
                    user.email = email
                    user.save()
            customer, created = self.get_or_create(user=user)
        return customer, created
